@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2014, Adrian Sampson.
+# Copyright 2015, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -32,6 +32,9 @@ Internally ``MediaFile`` uses ``MediaField`` descriptors to access the
 data from the tags. In turn ``MediaField`` uses a number of
 ``StorageStyle`` strategies to handle format specific logic.
 """
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
+
 import mutagen
 import mutagen.mp3
 import mutagen.oggopus
@@ -48,10 +51,10 @@ import math
 import struct
 import imghdr
 import os
-import logging
 import traceback
 import enum
 
+from beets import logging
 from beets.util import displayable_path
 
 
@@ -141,7 +144,7 @@ def _safe_cast(out_type, val):
             return False
 
     elif out_type == unicode:
-        if isinstance(val, str):
+        if isinstance(val, bytes):
             return val.decode('utf8', 'ignore')
         elif isinstance(val, unicode):
             return val
@@ -176,15 +179,15 @@ def _unpack_asf_image(data):
     of exceptions (out-of-bounds, etc.). We should clean this up
     sometime so that the failure modes are well-defined.
     """
-    type, size = struct.unpack_from("<bi", data)
+    type, size = struct.unpack_from(b'<bi', data)
     pos = 5
     mime = ""
-    while data[pos:pos + 2] != "\x00\x00":
+    while data[pos:pos + 2] != b'\x00\x00':
         mime += data[pos:pos + 2]
         pos += 2
     pos += 2
     description = ""
-    while data[pos:pos + 2] != "\x00\x00":
+    while data[pos:pos + 2] != b'\x00\x00':
         description += data[pos:pos + 2]
         pos += 2
     pos += 2
@@ -196,9 +199,9 @@ def _unpack_asf_image(data):
 def _pack_asf_image(mime, data, type=3, description=""):
     """Pack image data for a WM/Picture tag.
     """
-    tag_data = struct.pack("<bi", type, len(data))
-    tag_data += mime.encode("utf-16-le") + "\x00\x00"
-    tag_data += description.encode("utf-16-le") + "\x00\x00"
+    tag_data = struct.pack(b'<bi', type, len(data))
+    tag_data += mime.encode("utf-16-le") + b'\x00\x00'
+    tag_data += description.encode("utf-16-le") + b'\x00\x00'
     tag_data += data
     return tag_data
 
@@ -213,7 +216,7 @@ def _sc_decode(soundcheck):
     # characters of ASCII hex preceded by a space.
     try:
         soundcheck = soundcheck.replace(' ', '').decode('hex')
-        soundcheck = struct.unpack('!iiiiiiiiii', soundcheck)
+        soundcheck = struct.unpack(b'!iiiiiiiiii', soundcheck)
     except (struct.error, TypeError, UnicodeEncodeError):
         # SoundCheck isn't in the format we expect, so return default
         # values.
@@ -326,7 +329,11 @@ class Image(object):
         self.data = data
         self.desc = desc
         if isinstance(type, int):
-            type = list(ImageType)[type]
+            try:
+                type = list(ImageType)[type]
+            except IndexError:
+                log.debug(u"ignoring unknown image type index {0}", type)
+                type = ImageType.other
         self.type = type
 
     @property
@@ -395,8 +402,9 @@ class StorageStyle(object):
         self.float_places = float_places
 
         # Convert suffix to correct string type.
-        if self.suffix and self.as_type is unicode:
-            self.suffix = self.as_type(self.suffix)
+        if self.suffix and self.as_type is unicode \
+           and not isinstance(self.suffix, unicode):
+            self.suffix = self.suffix.decode('utf8')
 
     # Getter.
 
@@ -447,7 +455,7 @@ class StorageStyle(object):
             if isinstance(value, bool):
                 # Store bools as 1/0 instead of True/False.
                 value = unicode(int(bool(value)))
-            elif isinstance(value, str):
+            elif isinstance(value, bytes):
                 value = value.decode('utf8', 'ignore')
             else:
                 value = unicode(value)
@@ -560,7 +568,7 @@ class MP4StorageStyle(StorageStyle):
 
     def serialize(self, value):
         value = super(MP4StorageStyle, self).serialize(value)
-        if self.key.startswith('----:') and isinstance(value, unicode):
+        if self.key.startswith(b'----:') and isinstance(value, unicode):
             value = value.encode('utf8')
         return value
 
@@ -635,7 +643,7 @@ class MP4ImageStorageStyle(MP4ListStorageStyle):
     """Store images as MPEG-4 image atoms. Values are `Image` objects.
     """
     def __init__(self, **kwargs):
-        super(MP4ImageStorageStyle, self).__init__(key='covr', **kwargs)
+        super(MP4ImageStorageStyle, self).__init__(key=b'covr', **kwargs)
 
     def deserialize(self, data):
         return Image(data)
@@ -735,7 +743,7 @@ class MP3DescStorageStyle(MP3StorageStyle):
         # need to make a new frame?
         if not found:
             frame = mutagen.id3.Frames[self.key](
-                desc=str(self.description),
+                desc=bytes(self.description),
                 text=value,
                 encoding=3
             )
@@ -808,7 +816,7 @@ class MP3ImageStorageStyle(ListStorageStyle, MP3StorageStyle):
     """
     def __init__(self):
         super(MP3ImageStorageStyle, self).__init__(key='APIC')
-        self.as_type = str
+        self.as_type = bytes
 
     def deserialize(self, apic_frame):
         """Convert APIC frame into Image."""
@@ -877,7 +885,7 @@ class VorbisImageStorageStyle(ListStorageStyle):
         super(VorbisImageStorageStyle, self).__init__(
             key='metadata_block_picture'
         )
-        self.as_type = str
+        self.as_type = bytes
 
     def fetch(self, mutagen_file):
         images = []
@@ -990,7 +998,7 @@ class APEv2ImageStorageStyle(ListStorageStyle):
         for cover_type, cover_tag in self.TAG_NAMES.items():
             try:
                 frame = mutagen_file[cover_tag]
-                text_delimiter_index = frame.value.find('\x00')
+                text_delimiter_index = frame.value.find(b'\x00')
                 comment = frame.value[0:text_delimiter_index] \
                     if text_delimiter_index > 0 else None
                 image_data = frame.value[text_delimiter_index + 1:]
@@ -1007,7 +1015,7 @@ class APEv2ImageStorageStyle(ListStorageStyle):
         for image in values:
             image_type = image.type or ImageType.other
             comment = image.desc or ''
-            image_data = comment + "\x00" + image.data
+            image_data = comment.encode('utf8') + b'\x00' + image.data
             cover_tag = self.TAG_NAMES[image_type]
             mutagen_file[cover_tag] = image_data
 
@@ -1041,7 +1049,7 @@ class MediaField(object):
                          getting this property.
 
         """
-        self.out_type = kwargs.get('out_type', unicode)
+        self.out_type = kwargs.get(b'out_type', unicode)
         self._styles = styles
 
     def styles(self, mutagen_file):
@@ -1105,7 +1113,7 @@ class ListMediaField(MediaField):
         """Returns a ``MediaField`` descriptor that gets and sets the
         first item.
         """
-        options = {'out_type': self.out_type}
+        options = {b'out_type': self.out_type}
         return MediaField(*self._styles, **options)
 
 
@@ -1126,7 +1134,7 @@ class DateField(MediaField):
         storage styles do not return a value.
         """
         super(DateField, self).__init__(*date_styles)
-        year_style = kwargs.get('year', None)
+        year_style = kwargs.get(b'year', None)
         if year_style:
             self._year_field = MediaField(*year_style)
 
@@ -1238,18 +1246,31 @@ class DateItemField(MediaField):
 
 class CoverArtField(MediaField):
     """A descriptor that provides access to the *raw image data* for the
-    first image on a file. This is used for backwards compatibility: the
+    cover image on a file. This is used for backwards compatibility: the
     full `ImageListField` provides richer `Image` objects.
+
+    When there are multiple images we try to pick the most likely to be a front
+    cover.
     """
     def __init__(self):
         pass
 
     def __get__(self, mediafile, _):
-        try:
-            return mediafile.images[0].data
-        except IndexError:
+        candidates = mediafile.images
+        if candidates:
+            return self.guess_cover_image(candidates).data
+        else:
             return None
 
+    @staticmethod
+    def guess_cover_image(candidates):
+        if len(candidates) == 1:
+            return candidates[0]
+        try:
+            return next(c for c in candidates if c.type == ImageType.front)
+        except StopIteration:
+            return candidates[0]
+
     def __set__(self, mediafile, data):
         if data:
             mediafile.images = [Image(data=data)]
@@ -1313,7 +1334,7 @@ class MediaFile(object):
         try:
             self.mgfile = mutagen.File(path)
         except unreadable_exc as exc:
-            log.debug(u'header parsing failed: {0}'.format(unicode(exc)))
+            log.debug(u'header parsing failed: {0}', unicode(exc))
             raise UnreadableFileError(path)
         except IOError as exc:
             if type(exc) == IOError:
@@ -1326,7 +1347,7 @@ class MediaFile(object):
         except Exception as exc:
             # Isolate bugs in Mutagen.
             log.debug(traceback.format_exc())
-            log.error(u'uncaught Mutagen exception in open: {0}'.format(exc))
+            log.error(u'uncaught Mutagen exception in open: {0}', exc)
             raise MutagenError(path, exc)
 
         if self.mgfile is None:
@@ -1399,7 +1420,7 @@ class MediaFile(object):
             raise
         except Exception as exc:
             log.debug(traceback.format_exc())
-            log.error(u'uncaught Mutagen exception in save: {0}'.format(exc))
+            log.error(u'uncaught Mutagen exception in save: {0}', exc)
             raise MutagenError(self.path, exc)
 
     def delete(self):
@@ -1423,7 +1444,7 @@ class MediaFile(object):
         """
         for property, descriptor in cls.__dict__.items():
             if isinstance(descriptor, MediaField):
-                yield property
+                yield property.decode('utf8')
 
     @classmethod
     def readable_fields(cls):
@@ -1472,25 +1493,25 @@ class MediaFile(object):
 
     title = MediaField(
         MP3StorageStyle('TIT2'),
-        MP4StorageStyle("\xa9nam"),
+        MP4StorageStyle(b"\xa9nam"),
         StorageStyle('TITLE'),
         ASFStorageStyle('Title'),
     )
     artist = MediaField(
         MP3StorageStyle('TPE1'),
-        MP4StorageStyle("\xa9ART"),
+        MP4StorageStyle(b"\xa9ART"),
         StorageStyle('ARTIST'),
         ASFStorageStyle('Author'),
     )
     album = MediaField(
         MP3StorageStyle('TALB'),
-        MP4StorageStyle("\xa9alb"),
+        MP4StorageStyle(b"\xa9alb"),
         StorageStyle('ALBUM'),
         ASFStorageStyle('WM/AlbumTitle'),
     )
     genres = ListMediaField(
         MP3ListStorageStyle('TCON'),
-        MP4ListStorageStyle("\xa9gen"),
+        MP4ListStorageStyle(b"\xa9gen"),
         ListStorageStyle('GENRE'),
         ASFStorageStyle('WM/Genre'),
     )
@@ -1498,19 +1519,19 @@ class MediaFile(object):
 
     composer = MediaField(
         MP3StorageStyle('TCOM'),
-        MP4StorageStyle("\xa9wrt"),
+        MP4StorageStyle(b"\xa9wrt"),
         StorageStyle('COMPOSER'),
         ASFStorageStyle('WM/Composer'),
     )
     grouping = MediaField(
         MP3StorageStyle('TIT1'),
-        MP4StorageStyle("\xa9grp"),
+        MP4StorageStyle(b"\xa9grp"),
         StorageStyle('GROUPING'),
         ASFStorageStyle('WM/ContentGroupDescription'),
     )
     track = MediaField(
         MP3SlashPackStorageStyle('TRCK', pack_pos=0),
-        MP4TupleStorageStyle('trkn', index=0),
+        MP4TupleStorageStyle(b'trkn', index=0),
         StorageStyle('TRACK'),
         StorageStyle('TRACKNUMBER'),
         ASFStorageStyle('WM/TrackNumber'),
@@ -1518,7 +1539,7 @@ class MediaFile(object):
     )
     tracktotal = MediaField(
         MP3SlashPackStorageStyle('TRCK', pack_pos=1),
-        MP4TupleStorageStyle('trkn', index=1),
+        MP4TupleStorageStyle(b'trkn', index=1),
         StorageStyle('TRACKTOTAL'),
         StorageStyle('TRACKC'),
         StorageStyle('TOTALTRACKS'),
@@ -1527,7 +1548,7 @@ class MediaFile(object):
     )
     disc = MediaField(
         MP3SlashPackStorageStyle('TPOS', pack_pos=0),
-        MP4TupleStorageStyle('disk', index=0),
+        MP4TupleStorageStyle(b'disk', index=0),
         StorageStyle('DISC'),
         StorageStyle('DISCNUMBER'),
         ASFStorageStyle('WM/PartOfSet'),
@@ -1535,7 +1556,7 @@ class MediaFile(object):
     )
     disctotal = MediaField(
         MP3SlashPackStorageStyle('TPOS', pack_pos=1),
-        MP4TupleStorageStyle('disk', index=1),
+        MP4TupleStorageStyle(b'disk', index=1),
         StorageStyle('DISCTOTAL'),
         StorageStyle('DISCC'),
         StorageStyle('TOTALDISCS'),
@@ -1544,13 +1565,13 @@ class MediaFile(object):
     )
     lyrics = MediaField(
         MP3DescStorageStyle(key='USLT'),
-        MP4StorageStyle("\xa9lyr"),
+        MP4StorageStyle(b"\xa9lyr"),
         StorageStyle('LYRICS'),
         ASFStorageStyle('WM/Lyrics'),
     )
     comments = MediaField(
         MP3DescStorageStyle(key='COMM'),
-        MP4StorageStyle("\xa9cmt"),
+        MP4StorageStyle(b"\xa9cmt"),
         StorageStyle('DESCRIPTION'),
         StorageStyle('COMMENT'),
         ASFStorageStyle('WM/Comments'),
@@ -1558,111 +1579,111 @@ class MediaFile(object):
     )
     bpm = MediaField(
         MP3StorageStyle('TBPM'),
-        MP4StorageStyle('tmpo', as_type=int),
+        MP4StorageStyle(b'tmpo', as_type=int),
         StorageStyle('BPM'),
         ASFStorageStyle('WM/BeatsPerMinute'),
         out_type=int,
     )
     comp = MediaField(
         MP3StorageStyle('TCMP'),
-        MP4BoolStorageStyle('cpil'),
+        MP4BoolStorageStyle(b'cpil'),
         StorageStyle('COMPILATION'),
         ASFStorageStyle('WM/IsCompilation', as_type=bool),
         out_type=bool,
     )
     albumartist = MediaField(
         MP3StorageStyle('TPE2'),
-        MP4StorageStyle('aART'),
+        MP4StorageStyle(b'aART'),
         StorageStyle('ALBUM ARTIST'),
         StorageStyle('ALBUMARTIST'),
         ASFStorageStyle('WM/AlbumArtist'),
     )
     albumtype = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Album Type'),
-        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Album Type'),
+        MP4StorageStyle(b'----:com.apple.iTunes:MusicBrainz Album Type'),
         StorageStyle('MUSICBRAINZ_ALBUMTYPE'),
         ASFStorageStyle('MusicBrainz/Album Type'),
     )
     label = MediaField(
         MP3StorageStyle('TPUB'),
-        MP4StorageStyle('----:com.apple.iTunes:Label'),
-        MP4StorageStyle('----:com.apple.iTunes:publisher'),
+        MP4StorageStyle(b'----:com.apple.iTunes:Label'),
+        MP4StorageStyle(b'----:com.apple.iTunes:publisher'),
         StorageStyle('LABEL'),
         StorageStyle('PUBLISHER'),  # Traktor
         ASFStorageStyle('WM/Publisher'),
     )
     artist_sort = MediaField(
         MP3StorageStyle('TSOP'),
-        MP4StorageStyle("soar"),
+        MP4StorageStyle(b"soar"),
         StorageStyle('ARTISTSORT'),
         ASFStorageStyle('WM/ArtistSortOrder'),
     )
     albumartist_sort = MediaField(
         MP3DescStorageStyle(u'ALBUMARTISTSORT'),
-        MP4StorageStyle("soaa"),
+        MP4StorageStyle(b"soaa"),
         StorageStyle('ALBUMARTISTSORT'),
         ASFStorageStyle('WM/AlbumArtistSortOrder'),
     )
     asin = MediaField(
         MP3DescStorageStyle(u'ASIN'),
-        MP4StorageStyle("----:com.apple.iTunes:ASIN"),
+        MP4StorageStyle(b"----:com.apple.iTunes:ASIN"),
         StorageStyle('ASIN'),
         ASFStorageStyle('MusicBrainz/ASIN'),
     )
     catalognum = MediaField(
         MP3DescStorageStyle(u'CATALOGNUMBER'),
-        MP4StorageStyle("----:com.apple.iTunes:CATALOGNUMBER"),
+        MP4StorageStyle(b"----:com.apple.iTunes:CATALOGNUMBER"),
         StorageStyle('CATALOGNUMBER'),
         ASFStorageStyle('WM/CatalogNo'),
     )
     disctitle = MediaField(
         MP3StorageStyle('TSST'),
-        MP4StorageStyle("----:com.apple.iTunes:DISCSUBTITLE"),
+        MP4StorageStyle(b"----:com.apple.iTunes:DISCSUBTITLE"),
         StorageStyle('DISCSUBTITLE'),
         ASFStorageStyle('WM/SetSubTitle'),
     )
     encoder = MediaField(
         MP3StorageStyle('TENC'),
-        MP4StorageStyle("\xa9too"),
+        MP4StorageStyle(b"\xa9too"),
         StorageStyle('ENCODEDBY'),
         StorageStyle('ENCODER'),
         ASFStorageStyle('WM/EncodedBy'),
     )
     script = MediaField(
         MP3DescStorageStyle(u'Script'),
-        MP4StorageStyle("----:com.apple.iTunes:SCRIPT"),
+        MP4StorageStyle(b"----:com.apple.iTunes:SCRIPT"),
         StorageStyle('SCRIPT'),
         ASFStorageStyle('WM/Script'),
     )
     language = MediaField(
         MP3StorageStyle('TLAN'),
-        MP4StorageStyle("----:com.apple.iTunes:LANGUAGE"),
+        MP4StorageStyle(b"----:com.apple.iTunes:LANGUAGE"),
         StorageStyle('LANGUAGE'),
         ASFStorageStyle('WM/Language'),
     )
     country = MediaField(
         MP3DescStorageStyle('MusicBrainz Album Release Country'),
-        MP4StorageStyle("----:com.apple.iTunes:MusicBrainz "
-                        "Album Release Country"),
+        MP4StorageStyle(b"----:com.apple.iTunes:MusicBrainz "
+                        b"Album Release Country"),
         StorageStyle('RELEASECOUNTRY'),
         ASFStorageStyle('MusicBrainz/Album Release Country'),
     )
     albumstatus = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Album Status'),
-        MP4StorageStyle("----:com.apple.iTunes:MusicBrainz Album Status"),
+        MP4StorageStyle(b"----:com.apple.iTunes:MusicBrainz Album Status"),
         StorageStyle('MUSICBRAINZ_ALBUMSTATUS'),
         ASFStorageStyle('MusicBrainz/Album Status'),
     )
     media = MediaField(
         MP3StorageStyle('TMED'),
-        MP4StorageStyle("----:com.apple.iTunes:MEDIA"),
+        MP4StorageStyle(b"----:com.apple.iTunes:MEDIA"),
         StorageStyle('MEDIA'),
         ASFStorageStyle('WM/Media'),
     )
     albumdisambig = MediaField(
         # This tag mapping was invented for beets (not used by Picard, etc).
         MP3DescStorageStyle(u'MusicBrainz Album Comment'),
-        MP4StorageStyle("----:com.apple.iTunes:MusicBrainz Album Comment"),
+        MP4StorageStyle(b"----:com.apple.iTunes:MusicBrainz Album Comment"),
         StorageStyle('MUSICBRAINZ_ALBUMCOMMENT'),
         ASFStorageStyle('MusicBrainz/Album Comment'),
     )
@@ -1670,7 +1691,7 @@ class MediaFile(object):
     # Release date.
     date = DateField(
         MP3StorageStyle('TDRC'),
-        MP4StorageStyle("\xa9day"),
+        MP4StorageStyle(b"\xa9day"),
         StorageStyle('DATE'),
         ASFStorageStyle('WM/Year'),
         year=(StorageStyle('YEAR'),))
@@ -1682,7 +1703,7 @@ class MediaFile(object):
     # *Original* release date.
     original_date = DateField(
         MP3StorageStyle('TDOR'),
-        MP4StorageStyle('----:com.apple.iTunes:ORIGINAL YEAR'),
+        MP4StorageStyle(b'----:com.apple.iTunes:ORIGINAL YEAR'),
         StorageStyle('ORIGINALDATE'),
         ASFStorageStyle('WM/OriginalReleaseYear'))
 
@@ -1693,13 +1714,13 @@ class MediaFile(object):
     # Nonstandard metadata.
     artist_credit = MediaField(
         MP3DescStorageStyle(u'Artist Credit'),
-        MP4StorageStyle("----:com.apple.iTunes:Artist Credit"),
+        MP4StorageStyle(b"----:com.apple.iTunes:Artist Credit"),
         StorageStyle('ARTIST_CREDIT'),
         ASFStorageStyle('beets/Artist Credit'),
     )
     albumartist_credit = MediaField(
         MP3DescStorageStyle(u'Album Artist Credit'),
-        MP4StorageStyle("----:com.apple.iTunes:Album Artist Credit"),
+        MP4StorageStyle(b"----:com.apple.iTunes:Album Artist Credit"),
         StorageStyle('ALBUMARTIST_CREDIT'),
         ASFStorageStyle('beets/Album Artist Credit'),
     )
@@ -1713,31 +1734,31 @@ class MediaFile(object):
     # MusicBrainz IDs.
     mb_trackid = MediaField(
         MP3UFIDStorageStyle(owner='http://musicbrainz.org'),
-        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Track Id'),
+        MP4StorageStyle(b'----:com.apple.iTunes:MusicBrainz Track Id'),
         StorageStyle('MUSICBRAINZ_TRACKID'),
         ASFStorageStyle('MusicBrainz/Track Id'),
     )
     mb_albumid = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Album Id'),
-        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Album Id'),
+        MP4StorageStyle(b'----:com.apple.iTunes:MusicBrainz Album Id'),
         StorageStyle('MUSICBRAINZ_ALBUMID'),
         ASFStorageStyle('MusicBrainz/Album Id'),
     )
     mb_artistid = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Artist Id'),
-        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Artist Id'),
+        MP4StorageStyle(b'----:com.apple.iTunes:MusicBrainz Artist Id'),
         StorageStyle('MUSICBRAINZ_ARTISTID'),
         ASFStorageStyle('MusicBrainz/Artist Id'),
     )
     mb_albumartistid = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Album Artist Id'),
-        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Album Artist Id'),
+        MP4StorageStyle(b'----:com.apple.iTunes:MusicBrainz Album Artist Id'),
         StorageStyle('MUSICBRAINZ_ALBUMARTISTID'),
         ASFStorageStyle('MusicBrainz/Album Artist Id'),
     )
     mb_releasegroupid = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Release Group Id'),
-        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Release Group Id'),
+        MP4StorageStyle(b'----:com.apple.iTunes:MusicBrainz Release Group Id'),
         StorageStyle('MUSICBRAINZ_RELEASEGROUPID'),
         ASFStorageStyle('MusicBrainz/Release Group Id'),
     )
@@ -1745,13 +1766,13 @@ class MediaFile(object):
     # Acoustid fields.
     acoustid_fingerprint = MediaField(
         MP3DescStorageStyle(u'Acoustid Fingerprint'),
-        MP4StorageStyle('----:com.apple.iTunes:Acoustid Fingerprint'),
+        MP4StorageStyle(b'----:com.apple.iTunes:Acoustid Fingerprint'),
         StorageStyle('ACOUSTID_FINGERPRINT'),
         ASFStorageStyle('Acoustid/Fingerprint'),
     )
     acoustid_id = MediaField(
         MP3DescStorageStyle(u'Acoustid Id'),
-        MP4StorageStyle('----:com.apple.iTunes:Acoustid Id'),
+        MP4StorageStyle(b'----:com.apple.iTunes:Acoustid Id'),
         StorageStyle('ACOUSTID_ID'),
         ASFStorageStyle('Acoustid/Id'),
     )
@@ -1772,11 +1793,11 @@ class MediaFile(object):
             id3_lang='eng'
         ),
         MP4StorageStyle(
-            '----:com.apple.iTunes:replaygain_track_gain',
+            b'----:com.apple.iTunes:replaygain_track_gain',
             float_places=2, suffix=b' dB'
         ),
         MP4SoundCheckStorageStyle(
-            '----:com.apple.iTunes:iTunNORM',
+            b'----:com.apple.iTunes:iTunNORM',
             index=0
         ),
         StorageStyle(
@@ -1799,7 +1820,7 @@ class MediaFile(object):
             float_places=2, suffix=u' dB'
         ),
         MP4SoundCheckStorageStyle(
-            '----:com.apple.iTunes:iTunNORM',
+            b'----:com.apple.iTunes:iTunNORM',
             index=1
         ),
         StorageStyle(
@@ -1827,11 +1848,11 @@ class MediaFile(object):
             id3_lang='eng'
         ),
         MP4StorageStyle(
-            '----:com.apple.iTunes:replaygain_track_peak',
+            b'----:com.apple.iTunes:replaygain_track_peak',
             float_places=6
         ),
         MP4SoundCheckStorageStyle(
-            '----:com.apple.iTunes:iTunNORM',
+            b'----:com.apple.iTunes:iTunNORM',
             index=1
         ),
         StorageStyle(u'REPLAYGAIN_TRACK_PEAK', float_places=6),
@@ -1848,7 +1869,7 @@ class MediaFile(object):
             float_places=6
         ),
         MP4StorageStyle(
-            '----:com.apple.iTunes:replaygain_album_peak',
+            b'----:com.apple.iTunes:replaygain_album_peak',
             float_places=6
         ),
         StorageStyle(u'REPLAYGAIN_ALBUM_PEAK', float_places=6),
@@ -1858,7 +1879,7 @@ class MediaFile(object):
 
     initial_key = MediaField(
         MP3StorageStyle('TKEY'),
-        MP4StorageStyle('----:com.apple.iTunes:initialkey'),
+        MP4StorageStyle(b'----:com.apple.iTunes:initialkey'),
         StorageStyle('INITIALKEY'),
         ASFStorageStyle('INITIALKEY'),
     )
