@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2013, Pedro Silva.
+# Copyright 2015, Pedro Silva.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,91 +14,16 @@
 
 """List duplicate tracks or albums.
 """
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
+
 import shlex
-import logging
 
 from beets.plugins import BeetsPlugin
-from beets.ui import decargs, print_obj, vararg_callback, Subcommand, UserError
+from beets.ui import decargs, print_, vararg_callback, Subcommand, UserError
 from beets.util import command_output, displayable_path, subprocess
 
 PLUGIN = 'duplicates'
-log = logging.getLogger('beets')
-
-
-def _process_item(item, lib, copy=False, move=False, delete=False,
-                  tag=False, format=None):
-    """Process Item `item` in `lib`.
-    """
-    if copy:
-        item.move(basedir=copy, copy=True)
-        item.store()
-    if move:
-        item.move(basedir=move, copy=False)
-        item.store()
-    if delete:
-        item.remove(delete=True)
-    if tag:
-        try:
-            k, v = tag.split('=')
-        except:
-            raise UserError('%s: can\'t parse k=v tag: %s' % (PLUGIN, tag))
-        setattr(k, v)
-        item.store()
-    print_obj(item, lib, fmt=format)
-
-
-def _checksum(item, prog):
-    """Run external `prog` on file path associated with `item`, cache
-    output as flexattr on a key that is the name of the program, and
-    return the key, checksum tuple.
-    """
-    args = [p.format(file=item.path) for p in shlex.split(prog)]
-    key = args[0]
-    checksum = getattr(item, key, False)
-    if not checksum:
-        log.debug(u'{0}: key {1} on item {2} not cached: computing checksum'
-                  .format(PLUGIN, key, displayable_path(item.path)))
-        try:
-            checksum = command_output(args)
-            setattr(item, key, checksum)
-            item.store()
-            log.debug(u'{)}: computed checksum for {1} using {2}'
-                      .format(PLUGIN, item.title, key))
-        except subprocess.CalledProcessError as e:
-            log.debug(u'{0}: failed to checksum {1}: {2}'
-                      .format(PLUGIN, displayable_path(item.path), e))
-    else:
-        log.debug(u'{0}: key {1} on item {2} cached: not computing checksum'
-                  .format(PLUGIN, key, displayable_path(item.path)))
-    return key, checksum
-
-
-def _group_by(objs, keys):
-    """Return a dictionary with keys arbitrary concatenations of attributes and
-    values lists of objects (Albums or Items) with those keys.
-    """
-    import collections
-    counts = collections.defaultdict(list)
-    for obj in objs:
-        values = [getattr(obj, k, None) for k in keys]
-        values = [v for v in values if v not in (None, '')]
-        if values:
-            key = '\001'.join(values)
-            counts[key].append(obj)
-        else:
-            log.debug(u'{0}: all keys {1} on item {2} are null: skipping'
-                      .format(PLUGIN, str(keys), displayable_path(obj.path)))
-
-    return counts
-
-
-def _duplicates(objs, keys, full):
-    """Generate triples of keys, duplicate counts, and constituent objects.
-    """
-    offset = 0 if full else 1
-    for k, objs in _group_by(objs, keys).iteritems():
-        if len(objs) > 1:
-            yield (k, len(objs) - offset, objs[offset:])
 
 
 class DuplicatesPlugin(BeetsPlugin):
@@ -112,6 +37,7 @@ class DuplicatesPlugin(BeetsPlugin):
             'count': False,
             'album': False,
             'full': False,
+            'strict': False,
             'path': False,
             'keys': ['mb_trackid', 'mb_albumid'],
             'checksum': None,
@@ -124,17 +50,6 @@ class DuplicatesPlugin(BeetsPlugin):
         self._command = Subcommand('duplicates',
                                    help=__doc__,
                                    aliases=['dup'])
-
-        self._command.parser.add_option('-f', '--format', dest='format',
-                                        action='store', type='string',
-                                        help='print with custom format',
-                                        metavar='FMT')
-
-        self._command.parser.add_option('-a', '--album', dest='album',
-                                        action='store_true',
-                                        help='show duplicate albums instead of'
-                                        ' tracks')
-
         self._command.parser.add_option('-c', '--count', dest='count',
                                         action='store_true',
                                         help='show duplicate counts')
@@ -154,6 +69,11 @@ class DuplicatesPlugin(BeetsPlugin):
                                         help='show all versions of duplicate'
                                         ' tracks or albums')
 
+        self._command.parser.add_option('-s', '--strict', dest='strict',
+                                        action='store_true',
+                                        help='report duplicates only if all'
+                                        ' attributes are set')
+
         self._command.parser.add_option('-k', '--keys', dest='keys',
                                         action='callback', metavar='KEY1 KEY2',
                                         callback=vararg_callback,
@@ -167,15 +87,11 @@ class DuplicatesPlugin(BeetsPlugin):
                                         action='store', metavar='DEST',
                                         help='copy items to dest')
 
-        self._command.parser.add_option('-p', '--path', dest='path',
-                                        action='store_true',
-                                        help='print paths for matched items or'
-                                        ' albums')
-
         self._command.parser.add_option('-t', '--tag', dest='tag',
                                         action='store',
                                         help='tag matched items with \'k=v\''
                                         ' attribute')
+        self._command.parser.add_all_common_options()
 
     def commands(self):
 
@@ -184,6 +100,7 @@ class DuplicatesPlugin(BeetsPlugin):
             fmt = self.config['format'].get()
             album = self.config['album'].get(bool)
             full = self.config['full'].get(bool)
+            strict = self.config['strict'].get(bool)
             keys = self.config['keys'].get()
             checksum = self.config['checksum'].get()
             copy = self.config['copy'].get()
@@ -214,20 +131,102 @@ class DuplicatesPlugin(BeetsPlugin):
                         'duplicates: "checksum" option must be a command'
                     )
                 for i in items:
-                    k, _ = _checksum(i, checksum)
+                    k, _ = self._checksum(i, checksum)
                 keys = [k]
 
-            for obj_id, obj_count, objs in _duplicates(items,
-                                                       keys=keys,
-                                                       full=full):
+            for obj_id, obj_count, objs in self._duplicates(items,
+                                                            keys=keys,
+                                                            full=full,
+                                                            strict=strict):
                 if obj_id:  # Skip empty IDs.
                     for o in objs:
-                        _process_item(o, lib,
-                                      copy=copy,
-                                      move=move,
-                                      delete=delete,
-                                      tag=tag,
-                                      format=fmt.format(obj_count))
+                        self._process_item(o, lib,
+                                           copy=copy,
+                                           move=move,
+                                           delete=delete,
+                                           tag=tag,
+                                           fmt=fmt.format(obj_count))
 
         self._command.func = _dup
         return [self._command]
+
+    def _process_item(self, item, lib, copy=False, move=False, delete=False,
+                      tag=False, fmt=''):
+        """Process Item `item` in `lib`.
+        """
+        if copy:
+            item.move(basedir=copy, copy=True)
+            item.store()
+        if move:
+            item.move(basedir=move, copy=False)
+            item.store()
+        if delete:
+            item.remove(delete=True)
+        if tag:
+            try:
+                k, v = tag.split('=')
+            except:
+                raise UserError('%s: can\'t parse k=v tag: %s' % (PLUGIN, tag))
+            setattr(k, v)
+            item.store()
+        print_(format(item, fmt))
+
+    def _checksum(self, item, prog):
+        """Run external `prog` on file path associated with `item`, cache
+        output as flexattr on a key that is the name of the program, and
+        return the key, checksum tuple.
+        """
+        args = [p.format(file=item.path) for p in shlex.split(prog)]
+        key = args[0]
+        checksum = getattr(item, key, False)
+        if not checksum:
+            self._log.debug(u'key {0} on item {1} not cached:'
+                            'computing checksum',
+                            key, displayable_path(item.path))
+            try:
+                checksum = command_output(args)
+                setattr(item, key, checksum)
+                item.store()
+                self._log.debug(u'computed checksum for {0} using {1}',
+                                item.title, key)
+            except subprocess.CalledProcessError as e:
+                self._log.debug(u'failed to checksum {0}: {1}',
+                                displayable_path(item.path), e)
+        else:
+            self._log.debug(u'key {0} on item {1} cached:'
+                            'not computing checksum',
+                            key, displayable_path(item.path))
+        return key, checksum
+
+    def _group_by(self, objs, keys, strict):
+        """Return a dictionary with keys arbitrary concatenations of attributes and
+        values lists of objects (Albums or Items) with those keys.
+
+        If strict, all attributes must be defined for a duplicate match.
+        """
+        import collections
+        counts = collections.defaultdict(list)
+        for obj in objs:
+            values = [getattr(obj, k, None) for k in keys]
+            values = [v for v in values if v not in (None, '')]
+            if strict and len(values) < len(keys):
+                self._log.debug(u'some keys {0} on item {1} are null or empty:'
+                                ' skipping',
+                                keys, displayable_path(obj.path))
+            elif (not strict and not len(values)):
+                self._log.debug(u'all keys {0} on item {1} are null or empty:'
+                                ' skipping',
+                                keys, displayable_path(obj.path))
+            else:
+                key = tuple(values)
+                counts[key].append(obj)
+
+        return counts
+
+    def _duplicates(self, objs, keys, full, strict):
+        """Generate triples of keys, duplicate counts, and constituent objects.
+        """
+        offset = 0 if full else 1
+        for k, objs in self._group_by(objs, keys, strict).iteritems():
+            if len(objs) > 1:
+                yield (k, len(objs) - offset, objs[offset:])
