@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2015, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,8 +14,10 @@
 
 """Fetch a variety of acoustic metrics from The Echo Nest.
 """
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
+
 import time
-import logging
 import socket
 import os
 import tempfile
@@ -28,8 +30,6 @@ import pyechonest
 import pyechonest.song
 import pyechonest.track
 
-log = logging.getLogger('beets')
-
 # If a request at the EchoNest fails, we want to retry the request RETRIES
 # times and wait between retries for RETRY_INTERVAL seconds.
 RETRIES = 10
@@ -118,7 +118,7 @@ def similar(lib, src_item, threshold=0.15, fmt='${difference}: ${path}'):
             d = diff(item, src_item)
             if d < threshold:
                 s = fmt.replace('${difference}', '{:2.2f}'.format(d))
-                ui.print_obj(item, lib, s)
+                ui.print_(format(item, s))
 
 
 class EchonestMetadataPlugin(plugins.BeetsPlugin):
@@ -135,9 +135,10 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
             'truncate': True,
         })
         self.config.add(ATTRIBUTES)
+        self.config['apikey'].redact = True
 
         pyechonest.config.ECHO_NEST_API_KEY = \
-            config['echonest']['apikey'].get(unicode)
+            self.config['apikey'].get(unicode)
 
         if self.config['auto']:
             self.import_stages = [self.imported]
@@ -153,31 +154,30 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
             except pyechonest.util.EchoNestAPIError as e:
                 if e.code == 3:
                     # reached access limit per minute
-                    log.debug(u'echonest: rate-limited on try {0}; '
-                              u'waiting {1} seconds'
-                              .format(i + 1, RETRY_INTERVAL))
+                    self._log.debug(u'rate-limited on try {0}; waiting {1} '
+                                    u'seconds', i + 1, RETRY_INTERVAL)
                     time.sleep(RETRY_INTERVAL)
                 elif e.code == 5:
                     # specified identifier does not exist
                     # no use in trying again.
-                    log.debug(u'echonest: {0}'.format(e))
+                    self._log.debug(u'{0}', e)
                     return None
                 else:
-                    log.error(u'echonest: {0}'.format(e.args[0][0]))
+                    self._log.error(u'{0}', e.args[0][0])
                     return None
             except (pyechonest.util.EchoNestIOError, socket.error) as e:
-                log.warn(u'echonest: IO error: {0}'.format(e))
+                self._log.warn(u'IO error: {0}', e)
                 time.sleep(RETRY_INTERVAL)
             except Exception as e:
                 # there was an error analyzing the track, status: error
-                log.debug(u'echonest: {0}'.format(e))
+                self._log.debug(u'{0}', e)
                 return None
             else:
                 break
         else:
             # If we exited the loop without breaking, then we used up all
             # our allotted retries.
-            log.error(u'echonest request failed repeatedly')
+            self._log.error(u'request failed repeatedly')
             return None
         return result
 
@@ -188,7 +188,7 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         seconds, it's considered a match.
         """
         if not songs:
-            log.debug(u'echonest: no songs found')
+            self._log.debug(u'no songs found')
             return
 
         pick = None
@@ -226,13 +226,13 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         # Look up the Echo Nest ID based on the MBID.
         else:
             if not item.mb_trackid:
-                log.debug(u'echonest: no ID available')
+                self._log.debug(u'no ID available')
                 return
             mbid = 'musicbrainz:track:{0}'.format(item.mb_trackid)
             track = self._echofun(pyechonest.track.track_from_id,
                                   identifier=mbid)
             if not track:
-                log.debug(u'echonest: lookup by MBID failed')
+                self._log.debug(u'lookup by MBID failed')
                 return
             enid = track.song_id
 
@@ -267,13 +267,13 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         source = item.path
         tmp = None
         if item.format not in ALLOWED_FORMATS:
-            if config['echonest']['convert']:
+            if self.config['convert']:
                 tmp = source = self.convert(source)
             if not tmp:
                 return
 
         if os.stat(source).st_size > UPLOAD_MAX_SIZE:
-            if config['echonest']['truncate']:
+            if self.config['truncate']:
                 source = self.truncate(source)
                 if tmp is not None:
                     util.remove(tmp)
@@ -289,13 +289,12 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         pending.
         This is stolen from Jakob Schnitzers convert plugin.
         """
-        fd, dest = tempfile.mkstemp(u'.ogg')
+        fd, dest = tempfile.mkstemp(b'.ogg')
         os.close(fd)
 
-        log.info(u'echonest: encoding {0} to {1}'.format(
-            util.displayable_path(source),
-            util.displayable_path(dest),
-        ))
+        self._log.info(u'encoding {0} to {1}',
+                       util.displayable_path(source),
+                       util.displayable_path(dest))
 
         opts = []
         for arg in CONVERT_COMMAND.split():
@@ -306,13 +305,11 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         try:
             util.command_output(opts)
         except (OSError, subprocess.CalledProcessError) as exc:
-            log.debug(u'echonest: encode failed: {0}'.format(exc))
+            self._log.debug(u'encode failed: {0}', exc)
             util.remove(dest)
             return
 
-        log.info(u'echonest: finished encoding {0}'.format(
-            util.displayable_path(source))
-        )
+        self._log.info(u'finished encoding {0}', util.displayable_path(source))
         return dest
 
     def truncate(self, source):
@@ -320,10 +317,9 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         fd, dest = tempfile.mkstemp(u'.ogg')
         os.close(fd)
 
-        log.info(u'echonest: truncating {0} to {1}'.format(
-            util.displayable_path(source),
-            util.displayable_path(dest),
-        ))
+        self._log.info(u'truncating {0} to {1}',
+                       util.displayable_path(source),
+                       util.displayable_path(dest))
 
         opts = []
         for arg in TRUNCATE_COMMAND.split():
@@ -334,13 +330,11 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         try:
             util.command_output(opts)
         except (OSError, subprocess.CalledProcessError) as exc:
-            log.debug(u'echonest: truncate failed: {0}'.format(exc))
+            self._log.debug(u'truncate failed: {0}', exc)
             util.remove(dest)
             return
 
-        log.info(u'echonest: truncate encoding {0}'.format(
-            util.displayable_path(source))
-        )
+        self._log.info(u'truncate encoding {0}', util.displayable_path(source))
         return dest
 
     def analyze(self, item):
@@ -349,18 +343,18 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         """
         prepared = self.prepare_upload(item)
         if not prepared:
-            log.debug(u'echonest: could not prepare file for upload')
+            self._log.debug(u'could not prepare file for upload')
             return
 
         source, tmp = prepared
-        log.info(u'echonest: uploading file, please be patient')
+        self._log.info(u'uploading file, please be patient')
         track = self._echofun(pyechonest.track.track_from_filename,
                               filename=source)
         if tmp is not None:
             util.remove(tmp)
 
         if not track:
-            log.debug(u'echonest: failed to upload file')
+            self._log.debug(u'failed to upload file')
             return
 
         # Sometimes we have a track but no song. I guess this happens for
@@ -404,21 +398,18 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         # There are four different ways to get a song. Each method is a
         # callable that takes the Item as an argument.
         methods = [self.profile, self.search]
-        if config['echonest']['upload']:
+        if self.config['upload']:
             methods.append(self.analyze)
 
         # Try each method in turn.
         for method in methods:
             song = method(item)
             if song:
-                log.debug(
-                    u'echonest: got song through {0}: {1} - {2} [{3}]'.format(
-                        method.__name__,
-                        item.artist,
-                        item.title,
-                        song.get('duration'),
-                    )
-                )
+                self._log.debug(u'got song through {0}: {1} [{2}]',
+                                method.__name__,
+                                item,
+                                song.get('duration'),
+                                )
                 return song
 
     def apply_metadata(self, item, values, write=False):
@@ -429,7 +420,7 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         for k, v in values.iteritems():
             if k in ATTRIBUTES:
                 field = ATTRIBUTES[k]
-                log.debug(u'echonest: metadata: {0} = {1}'.format(field, v))
+                self._log.debug(u'metadata: {0} = {1}', field, v)
                 if field == 'bpm':
                     item[field] = int(v)
                 else:
@@ -441,7 +432,7 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
             item['initial_key'] = key
         if 'id' in values:
             enid = values['id']
-            log.debug(u'echonest: metadata: {0} = {1}'.format(ID_KEY, enid))
+            self._log.debug(u'metadata: {0} = {1}', ID_KEY, enid)
             item[ID_KEY] = enid
 
         # Write and save.
@@ -468,7 +459,7 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
         for field in ATTRIBUTES.values():
             if not item.get(field):
                 return True
-        log.info(u'echonest: no update required')
+        self._log.info(u'no update required')
         return False
 
     def commands(self):
@@ -483,8 +474,7 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
             self.config.set_args(opts)
             write = config['import']['write'].get(bool)
             for item in lib.items(ui.decargs(args)):
-                log.info(u'echonest: {0} - {1}'.format(item.artist,
-                                                       item.title))
+                self._log.info(u'{0}', item)
                 if self.config['force'] or self.requires_update(item):
                     song = self.fetch_song(item)
                     if song:
@@ -497,10 +487,7 @@ class EchonestMetadataPlugin(plugins.BeetsPlugin):
             '-t', '--threshold', dest='threshold', action='store',
             type='float', default=0.15, help='Set difference threshold'
         )
-        sim_cmd.parser.add_option(
-            '-f', '--format', action='store', default='${difference}: ${path}',
-            help='print with custom format'
-        )
+        sim_cmd.parser.add_format_option()
 
         def sim_func(lib, opts, args):
             self.config.set_args(opts)
