@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2015, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -12,6 +12,9 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
+
 """Gets genres for imported music based on Last.fm tags.
 
 Uses a provided whitelist file to determine which tags are valid genres.
@@ -20,10 +23,10 @@ and has been edited to remove some questionable entries.
 The scraper script used is available here:
 https://gist.github.com/1241307
 """
-import logging
 import pylast
 import os
 import yaml
+import traceback
 
 from beets import plugins
 from beets import ui
@@ -31,7 +34,6 @@ from beets.util import normpath, plurality
 from beets import config
 from beets import library
 
-log = logging.getLogger('beets')
 
 LASTFM = pylast.LastFMNetwork(api_key=plugins.LASTFM_KEY)
 
@@ -53,40 +55,8 @@ def deduplicate(seq):
     return [x for x in seq if x not in seen and not seen.add(x)]
 
 
-# Core genre identification routine.
-
-def _tags_for(obj, min_weight=None):
-    """Given a pylast entity (album or track), return a list of
-    tag names for that entity. Return an empty list if the entity is
-    not found or another error occurs.
-
-    If `min_weight` is specified, tags are filtered by weight.
-    """
-    try:
-        # Work around an inconsistency in pylast where
-        # Album.get_top_tags() does not return TopItem instances.
-        # https://code.google.com/p/pylast/issues/detail?id=85
-        if isinstance(obj, pylast.Album):
-            res = super(pylast.Album, obj).get_top_tags()
-        else:
-            res = obj.get_top_tags()
-    except PYLAST_EXCEPTIONS as exc:
-        log.debug(u'last.fm error: {0}'.format(exc))
-        return []
-
-    # Filter by weight (optionally).
-    if min_weight:
-        res = [el for el in res if (el.weight or 0) >= min_weight]
-
-    # Get strings from tags.
-    res = [el.item.get_name().lower() for el in res]
-
-    return res
-
-
 # Canonicalization tree processing.
 
-
 def flatten_tree(elem, path, branches):
     """Flatten nested lists/dictionaries into lists of strings
     (branches).
@@ -156,7 +126,7 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             wl_filename = WHITELIST
         if wl_filename:
             wl_filename = normpath(wl_filename)
-            with open(wl_filename, 'r') as f:
+            with open(wl_filename, b'r') as f:
                 for line in f:
                     line = line.decode('utf8').strip().lower()
                     if line and not line.startswith(u'#'):
@@ -225,7 +195,7 @@ class LastGenrePlugin(plugins.BeetsPlugin):
         can be found. Ex. 'Electronic, House, Dance'
         """
         min_weight = self.config['min_weight'].get(int)
-        return self._resolve_genres(_tags_for(lastfm_obj, min_weight))
+        return self._resolve_genres(self._tags_for(lastfm_obj, min_weight))
 
     def _is_allowed(self, genre):
         """Determine whether the genre is present in the whitelist,
@@ -371,9 +341,8 @@ class LastGenrePlugin(plugins.BeetsPlugin):
 
             for album in lib.albums(ui.decargs(args)):
                 album.genre, src = self._get_genre(album)
-                log.info(u'genre for album {0} - {1} ({2}): {3}'.format(
-                    album.albumartist, album.album, src, album.genre
-                ))
+                self._log.info(u'genre for album {0} ({1}): {0.genre}',
+                               album, src)
                 album.store()
 
                 for item in album.items():
@@ -382,9 +351,8 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                     if 'track' in self.sources:
                         item.genre, src = self._get_genre(item)
                         item.store()
-                        log.info(u'genre for track {0} - {1} ({2}): {3}'
-                                 .format(item.artist, item.title, src,
-                                         item.genre))
+                        self._log.info(u'genre for track {0} ({1}): {0.genre}',
+                                       item, src)
 
                     if write:
                         item.try_write()
@@ -397,20 +365,55 @@ class LastGenrePlugin(plugins.BeetsPlugin):
         if task.is_album:
             album = task.album
             album.genre, src = self._get_genre(album)
-            log.debug(u'added last.fm album genre ({0}): {1}'.format(
-                src, album.genre))
+            self._log.debug(u'added last.fm album genre ({0}): {1}',
+                            src, album.genre)
             album.store()
 
             if 'track' in self.sources:
                 for item in album.items():
                     item.genre, src = self._get_genre(item)
-                    log.debug(u'added last.fm item genre ({0}): {1}'.format(
-                        src, item.genre))
+                    self._log.debug(u'added last.fm item genre ({0}): {1}',
+                                    src, item.genre)
                     item.store()
 
         else:
             item = task.item
             item.genre, src = self._get_genre(item)
-            log.debug(u'added last.fm item genre ({0}): {1}'.format(
-                src, item.genre))
+            self._log.debug(u'added last.fm item genre ({0}): {1}',
+                            src, item.genre)
             item.store()
+
+    def _tags_for(self, obj, min_weight=None):
+        """Core genre identification routine.
+
+        Given a pylast entity (album or track), return a list of
+        tag names for that entity. Return an empty list if the entity is
+        not found or another error occurs.
+
+        If `min_weight` is specified, tags are filtered by weight.
+        """
+        # Work around an inconsistency in pylast where
+        # Album.get_top_tags() does not return TopItem instances.
+        # https://code.google.com/p/pylast/issues/detail?id=85
+        if isinstance(obj, pylast.Album):
+            obj = super(pylast.Album, obj)
+
+        try:
+            res = obj.get_top_tags()
+        except PYLAST_EXCEPTIONS as exc:
+            self._log.debug(u'last.fm error: {0}', exc)
+            return []
+        except Exception as exc:
+            # Isolate bugs in pylast.
+            self._log.debug(traceback.format_exc())
+            self._log.error('error in pylast library: {0}', exc)
+            return []
+
+        # Filter by weight (optionally).
+        if min_weight:
+            res = [el for el in res if (el.weight or 0) >= min_weight]
+
+        # Get strings from tags.
+        res = [el.item.get_name().lower() for el in res]
+
+        return res
