@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2014, Fabrice Laporte, Yevgeny Bezman, and Adrian Sampson.
+# Copyright 2015, Fabrice Laporte, Yevgeny Bezman, and Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -12,21 +12,23 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-import logging
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
+
 import subprocess
 import os
 import collections
 import itertools
 import sys
 import warnings
+import re
 
+from beets import logging
 from beets import ui
 from beets.plugins import BeetsPlugin
 from beets.util import syspath, command_output, displayable_path
 from beets import config
 
-log = logging.getLogger('beets.replaygain')
-
 
 # Utilities.
 
@@ -67,10 +69,12 @@ AlbumGain = collections.namedtuple("AlbumGain", "album_gain track_gains")
 class Backend(object):
     """An abstract class representing engine for calculating RG values.
     """
-    def __init__(self, config):
+
+    def __init__(self, config, log):
         """Initialize the backend with the configuration view for the
         plugin.
         """
+        self._log = log
 
     def compute_track_gain(self, items):
         raise NotImplementedError()
@@ -81,11 +85,166 @@ class Backend(object):
         raise NotImplementedError()
 
 
+# bsg1770gain backend
+class Bs1770gainBackend(Backend):
+    """bs1770gain is a loudness scanner compliant with ITU-R BS.1770 and
+    its flavors EBU R128, ATSC A/85 and Replaygain 2.0.
+    """
+
+    def __init__(self, config, log):
+        super(Bs1770gainBackend, self).__init__(config, log)
+        config.add({
+            'chunk_at': 5000,
+            'method': 'replaygain',
+        })
+        self.chunk_at = config['chunk_at'].as_number()
+        self.method = b'--' + bytes(config['method'].get(unicode))
+
+        cmd = b'bs1770gain'
+        try:
+            call([cmd, self.method])
+            self.command = cmd
+        except OSError:
+            raise FatalReplayGainError(
+                'Is bs1770gain installed? Is your method in config correct?'
+            )
+        if not self.command:
+            raise FatalReplayGainError(
+                'no replaygain command found: install bs1770gain'
+            )
+
+    def compute_track_gain(self, items):
+        """Computes the track gain of the given tracks, returns a list
+        of TrackGain objects.
+        """
+
+        output = self.compute_gain(items, False)
+        return output
+
+    def compute_album_gain(self, album):
+        """Computes the album gain of the given album, returns an
+        AlbumGain object.
+        """
+        # TODO: What should be done when not all tracks in the album are
+        # supported?
+
+        supported_items = album.items()
+        output = self.compute_gain(supported_items, True)
+
+        if not output:
+            raise ReplayGainError('no output from bs1770gain')
+        return AlbumGain(output[-1], output[:-1])
+
+    def isplitter(self, items, chunk_at):
+        """Break an iterable into chunks of at most size `chunk_at`,
+        generating lists for each chunk.
+        """
+        iterable = iter(items)
+        while True:
+            result = []
+            for i in range(chunk_at):
+                try:
+                    a = next(iterable)
+                except StopIteration:
+                    break
+                else:
+                    result.append(a)
+            if result:
+                yield result
+            else:
+                break
+
+    def compute_gain(self, items, is_album):
+        """Computes the track or album gain of a list of items, returns
+        a list of TrackGain objects.
+        When computing album gain, the last TrackGain object returned is
+        the album gain
+        """
+
+        if len(items) == 0:
+            return []
+
+        albumgaintot = 0.0
+        albumpeaktot = 0.0
+        returnchunks = []
+
+        # In the case of very large sets of music, we break the tracks
+        # into smaller chunks and process them one at a time. This
+        # avoids running out of memory.
+        if len(items) > self.chunk_at:
+            i = 0
+            for chunk in self.isplitter(items, self.chunk_at):
+                i += 1
+                returnchunk = self.compute_chunk_gain(chunk, is_album)
+                albumgaintot += returnchunk[-1].gain
+                albumpeaktot += returnchunk[-1].peak
+                returnchunks = returnchunks + returnchunk[0:-1]
+            returnchunks.append(Gain(albumgaintot / i, albumpeaktot / i))
+            return returnchunks
+        else:
+            return self.compute_chunk_gain(items, is_album)
+
+    def compute_chunk_gain(self, items, is_album):
+        """Compute ReplayGain values and return a list of results
+        dictionaries as given by `parse_tool_output`.
+        """
+        # Construct shell command.
+        cmd = [self.command]
+        cmd = cmd + [self.method]
+        cmd = cmd + [b'-it']
+
+        # Workaround for Windows: the underlying tool fails on paths
+        # with the \\?\ prefix, so we don't use it here. This
+        # prevents the backend from working with long paths.
+        args = cmd + [syspath(i.path, prefix=False) for i in items]
+
+        # Invoke the command.
+        self._log.debug("executing {0}", " ".join(map(displayable_path, args)))
+        output = call(args)
+
+        self._log.debug(u'analysis finished: {0}', output)
+        results = self.parse_tool_output(output,
+                                         len(items) + is_album)
+        self._log.debug(u'{0} items, {1} results', len(items), len(results))
+        return results
+
+    def parse_tool_output(self, text, num_lines):
+        """Given the  output from bs1770gain, parse the text and
+        return a list of dictionaries
+        containing information about each analyzed file.
+        """
+        out = []
+        data = text.decode('utf8', errors='ignore')
+        regex = re.compile(
+            ur'(\s{2,2}\[\d+\/\d+\].*?|\[ALBUM\].*?)'
+            '(?=\s{2,2}\[\d+\/\d+\]|\s{2,2}\[ALBUM\]'
+            ':|done\.\s)', re.DOTALL | re.UNICODE)
+        results = re.findall(regex, data)
+        for parts in results[0:num_lines]:
+            part = parts.split(b'\n')
+            if len(part) == 0:
+                self._log.debug('bad tool output: {0!r}', text)
+                raise ReplayGainError('bs1770gain failed')
+
+            try:
+                song = {
+                    'file': part[0],
+                    'gain': float((part[1].split('/'))[1].split('LU')[0]),
+                    'peak': float(part[2].split('/')[1]),
+                }
+            except IndexError:
+                self._log.info('bs1770gain reports (faulty file?): {}', parts)
+                continue
+
+            out.append(Gain(song['gain'], song['peak']))
+        return out
+
+
 # mpgain/aacgain CLI tool backend.
-
-
 class CommandBackend(Backend):
-    def __init__(self, config):
+
+    def __init__(self, config, log):
+        super(CommandBackend, self).__init__(config, log)
         config.add({
             'command': u"",
             'noclip': True,
@@ -103,9 +262,9 @@ class CommandBackend(Backend):
                 )
         else:
             # Check whether the program is in $PATH.
-            for cmd in ('mp3gain', 'aacgain'):
+            for cmd in (b'mp3gain', b'aacgain'):
                 try:
-                    call([cmd, '-v'])
+                    call([cmd, b'-v'])
                     self.command = cmd
                 except OSError:
                     pass
@@ -135,7 +294,7 @@ class CommandBackend(Backend):
 
         supported_items = filter(self.format_supported, album.items())
         if len(supported_items) != len(album.items()):
-            log.debug(u'replaygain: tracks are of unsupported format')
+            self._log.debug(u'tracks are of unsupported format')
             return AlbumGain(None, [])
 
         output = self.compute_gain(supported_items, True)
@@ -169,26 +328,22 @@ class CommandBackend(Backend):
         # tag-writing; this turns the mp3gain/aacgain tool into a gain
         # calculator rather than a tag manipulator because we take care
         # of changing tags ourselves.
-        cmd = [self.command, '-o', '-s', 's']
+        cmd = [self.command, b'-o', b'-s', b's']
         if self.noclip:
             # Adjust to avoid clipping.
-            cmd = cmd + ['-k']
+            cmd = cmd + [b'-k']
         else:
             # Disable clipping warning.
-            cmd = cmd + ['-c']
-        cmd = cmd + ['-a' if is_album else '-r']
-        cmd = cmd + ['-d', str(self.gain_offset)]
+            cmd = cmd + [b'-c']
+        cmd = cmd + [b'-d', bytes(self.gain_offset)]
         cmd = cmd + [syspath(i.path) for i in items]
 
-        log.debug(u'replaygain: analyzing {0} files'.format(len(items)))
-        log.debug(u"replaygain: executing {0}"
-                  .format(" ".join(map(displayable_path, cmd))))
+        self._log.debug(u'analyzing {0} files', len(items))
+        self._log.debug(u"executing {0}", " ".join(map(displayable_path, cmd)))
         output = call(cmd)
-        log.debug(u'replaygain: analysis finished')
-        results = self.parse_tool_output(output,
-                                         len(items) + (1 if is_album else 0))
-
-        return results
+        self._log.debug(u'analysis finished')
+        return self.parse_tool_output(output,
+                                      len(items) + (1 if is_album else 0))
 
     def parse_tool_output(self, text, num_lines):
         """Given the tab-delimited output from an invocation of mp3gain
@@ -196,10 +351,10 @@ class CommandBackend(Backend):
         containing information about each analyzed file.
         """
         out = []
-        for line in text.split('\n')[1:num_lines + 1]:
-            parts = line.split('\t')
-            if len(parts) != 6 or parts[0] == 'File':
-                log.debug(u'replaygain: bad tool output: {0}'.format(text))
+        for line in text.split(b'\n')[1:num_lines + 1]:
+            parts = line.split(b'\t')
+            if len(parts) != 6 or parts[0] == b'File':
+                self._log.debug(u'bad tool output: {0}', text)
                 raise ReplayGainError('mp3gain failed')
             d = {
                 'file': parts[0],
@@ -216,8 +371,10 @@ class CommandBackend(Backend):
 
 # GStreamer-based backend.
 
-class GStreamerBackend(object):
-    def __init__(self, config):
+class GStreamerBackend(Backend):
+
+    def __init__(self, config, log):
+        super(GStreamerBackend, self).__init__(config, log)
         self._import_gst()
 
         # Initialized a GStreamer pipeline of the form filesrc ->
@@ -468,7 +625,9 @@ class AudioToolsBackend(Backend):
     <http://audiotools.sourceforge.net/>`_ and its capabilities to read more
     file formats and compute ReplayGain values using it replaygain module.
     """
-    def __init__(self, config):
+
+    def __init__(self, config, log):
+        super(AudioToolsBackend, self).__init__(config, log)
         self._import_audiotools()
 
     def _import_audiotools(self):
@@ -548,14 +707,8 @@ class AudioToolsBackend(Backend):
         # be obtained from an audiofile instance.
         rg_track_gain, rg_track_peak = rg.title_gain(audiofile.to_pcm())
 
-        log.debug(
-            u'ReplayGain for track {0} - {1}: {2:.2f}, {3:.2f}'.format(
-                item.artist,
-                item.title,
-                rg_track_gain,
-                rg_track_peak
-            )
-        )
+        self._log.debug(u'ReplayGain for track {0} - {1}: {2:.2f}, {3:.2f}',
+                        item.artist, item.title, rg_track_gain, rg_track_peak)
         return Gain(gain=rg_track_gain, peak=rg_track_peak)
 
     def compute_album_gain(self, album):
@@ -563,12 +716,7 @@ class AudioToolsBackend(Backend):
 
         :rtype: :class:`AlbumGain`
         """
-        log.debug(
-            u'Analysing album {0} - {1}'.format(
-                album.albumartist,
-                album.album
-            )
-        )
+        self._log.debug(u'Analysing album {0}', album)
 
         # The first item is taken and opened to get the sample rate to
         # initialize the replaygain object. The object is used for all the
@@ -584,26 +732,14 @@ class AudioToolsBackend(Backend):
             track_gains.append(
                 Gain(gain=rg_track_gain, peak=rg_track_peak)
             )
-            log.debug(
-                u'ReplayGain for track {0} - {1}: {2:.2f}, {3:.2f}'.format(
-                    item.artist,
-                    item.title,
-                    rg_track_gain,
-                    rg_track_peak
-                )
-            )
+            self._log.debug(u'ReplayGain for track {0}: {1:.2f}, {2:.2f}',
+                            item, rg_track_gain, rg_track_peak)
 
         # After getting the values for all tracks, it's possible to get the
         # album values.
         rg_album_gain, rg_album_peak = rg.album_gain()
-        log.debug(
-            u'ReplayGain for Album {0} - {1}: {2:.2f}, {3:.2f}'.format(
-                album.albumartist,
-                album.album,
-                rg_album_gain,
-                rg_album_peak
-            )
-        )
+        self._log.debug(u'ReplayGain for album {0}: {1:.2f}, {2:.2f}',
+                        album, rg_album_gain, rg_album_peak)
 
         return AlbumGain(
             Gain(gain=rg_album_gain, peak=rg_album_peak),
@@ -618,14 +754,14 @@ class ReplayGainPlugin(BeetsPlugin):
     """
 
     backends = {
-        "command":   CommandBackend,
+        "command": CommandBackend,
         "gstreamer": GStreamerBackend,
-        "audiotools": AudioToolsBackend
+        "audiotools": AudioToolsBackend,
+        "bs1770gain": Bs1770gainBackend
     }
 
     def __init__(self):
         super(ReplayGainPlugin, self).__init__()
-        self.import_stages = [self.imported]
 
         # default backend is 'command' for backward-compatibility.
         self.config.add({
@@ -636,7 +772,6 @@ class ReplayGainPlugin(BeetsPlugin):
         })
 
         self.overwrite = self.config['overwrite'].get(bool)
-        self.automatic = self.config['auto'].get(bool)
         backend_name = self.config['backend'].get(unicode)
         if backend_name not in self.backends:
             raise ui.UserError(
@@ -647,9 +782,13 @@ class ReplayGainPlugin(BeetsPlugin):
                 )
             )
 
+        # On-import analysis.
+        if self.config['auto']:
+            self.import_stages = [self.imported]
+
         try:
             self.backend_instance = self.backends[backend_name](
-                self.config
+                self.config, self._log
             )
         except (ReplayGainError, FatalReplayGainError) as e:
             raise ui.UserError(
@@ -674,19 +813,16 @@ class ReplayGainPlugin(BeetsPlugin):
         item.rg_track_peak = track_gain.peak
         item.store()
 
-        log.debug(u'replaygain: applied track gain {0}, peak {1}'.format(
-            item.rg_track_gain,
-            item.rg_track_peak
-        ))
+        self._log.debug(u'applied track gain {0}, peak {1}',
+                        item.rg_track_gain, item.rg_track_peak)
 
     def store_album_gain(self, album, album_gain):
         album.rg_album_gain = album_gain.gain
         album.rg_album_peak = album_gain.peak
         album.store()
 
-        log.debug(u'replaygain: applied album gain {0}, peak {1}'.format(
-            album.rg_album_gain,
-            album.rg_album_peak))
+        self._log.debug(u'applied album gain {0}, peak {1}',
+                        album.rg_album_gain, album.rg_album_peak)
 
     def handle_album(self, album, write):
         """Compute album and track replay gain store it in all of the
@@ -697,21 +833,17 @@ class ReplayGainPlugin(BeetsPlugin):
         items, nothing is done.
         """
         if not self.album_requires_gain(album):
-            log.info(u'Skipping album {0} - {1}'.format(album.albumartist,
-                                                        album.album))
+            self._log.info(u'Skipping album {0}', album)
             return
 
-        log.info(u'analyzing {0} - {1}'.format(album.albumartist,
-                                               album.album))
+        self._log.info(u'analyzing {0}', album)
 
         try:
             album_gain = self.backend_instance.compute_album_gain(album)
             if len(album_gain.track_gains) != len(album.items()):
                 raise ReplayGainError(
                     u"ReplayGain backend failed "
-                    u"for some tracks in album {0} - {1}".format(
-                        album.albumartist, album.album
-                    )
+                    u"for some tracks in album {0}".format(album)
                 )
 
             self.store_album_gain(album, album_gain.album_gain)
@@ -721,7 +853,7 @@ class ReplayGainPlugin(BeetsPlugin):
                 if write:
                     item.try_write()
         except ReplayGainError as e:
-            log.info(u"ReplayGain error: {0}".format(e))
+            self._log.info(u"ReplayGain error: {0}", e)
         except FatalReplayGainError as e:
             raise ui.UserError(
                 u"Fatal replay gain error: {0}".format(e)
@@ -735,27 +867,23 @@ class ReplayGainPlugin(BeetsPlugin):
         in the item, nothing is done.
         """
         if not self.track_requires_gain(item):
-            log.info(u'Skipping track {0} - {1}'
-                     .format(item.artist, item.title))
+            self._log.info(u'Skipping track {0}', item)
             return
 
-        log.info(u'analyzing {0} - {1}'
-                 .format(item.artist, item.title))
+        self._log.info(u'analyzing {0}', item)
 
         try:
             track_gains = self.backend_instance.compute_track_gain([item])
             if len(track_gains) != 1:
                 raise ReplayGainError(
-                    u"ReplayGain backend failed for track {0} - {1}".format(
-                        item.artist, item.title
-                    )
+                    u"ReplayGain backend failed for track {0}".format(item)
                 )
 
             self.store_track_gain(item, track_gains[0])
             if write:
                 item.try_write()
         except ReplayGainError as e:
-            log.info(u"ReplayGain error: {0}".format(e))
+            self._log.info(u"ReplayGain error: {0}", e)
         except FatalReplayGainError as e:
             raise ui.UserError(
                 u"Fatal replay gain error: {0}".format(e)
@@ -764,11 +892,6 @@ class ReplayGainPlugin(BeetsPlugin):
     def imported(self, session, task):
         """Add replay gain info to items or albums of ``task``.
         """
-        if not self.automatic:
-            return
-
-        log.setLevel(logging.WARN)
-
         if task.is_album:
             self.handle_album(task.album, False)
         else:
@@ -778,7 +901,7 @@ class ReplayGainPlugin(BeetsPlugin):
         """Return the "replaygain" ui subcommand.
         """
         def func(lib, opts, args):
-            log.setLevel(logging.INFO)
+            self._log.setLevel(logging.INFO)
 
             write = config['import']['write'].get(bool)
 
@@ -791,7 +914,6 @@ class ReplayGainPlugin(BeetsPlugin):
                     self.handle_track(item, write)
 
         cmd = ui.Subcommand('replaygain', help='analyze for ReplayGain')
-        cmd.parser.add_option('-a', '--album', action='store_true',
-                              help='analyze albums instead of tracks')
+        cmd.parser.add_album_option()
         cmd.func = func
         return [cmd]
