@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2013, Blemjhoo Tezoulbr <baobab@heresiarch.info>.
+# Copyright 2015, Blemjhoo Tezoulbr <baobab@heresiarch.info>.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,8 +14,10 @@
 
 """ Clears tag fields in media files."""
 
+from __future__ import (division, absolute_import, print_function,
+                        unicode_literals)
+
 import re
-import logging
 from beets.plugins import BeetsPlugin
 from beets.mediafile import MediaFile
 from beets.importer import action
@@ -24,8 +26,6 @@ from beets.util import confit
 __author__ = 'baobab@heresiarch.info'
 __version__ = '0.10'
 
-log = logging.getLogger('beets')
-
 
 class ZeroPlugin(BeetsPlugin):
 
@@ -48,11 +48,11 @@ class ZeroPlugin(BeetsPlugin):
 
         for field in self.config['fields'].as_str_seq():
             if field in ('id', 'path', 'album_id'):
-                log.warn(u'[zero] field \'{0}\' ignored, zeroing '
-                         u'it would be dangerous'.format(field))
+                self._log.warn(u'field \'{0}\' ignored, zeroing '
+                               u'it would be dangerous', field)
                 continue
             if field not in MediaFile.fields():
-                log.error(u'[zero] invalid field: {0}'.format(field))
+                self._log.error(u'invalid field: {0}', field)
                 continue
 
             try:
@@ -64,7 +64,7 @@ class ZeroPlugin(BeetsPlugin):
     def import_task_choice_event(self, session, task):
         """Listen for import_task_choice event."""
         if task.choice_flag == action.ASIS and not self.warned:
-            log.warn(u'[zero] cannot zero in \"as-is\" mode')
+            self._log.warn(u'cannot zero in \"as-is\" mode')
             self.warned = True
         # TODO request write in as-is mode
 
@@ -85,7 +85,7 @@ class ZeroPlugin(BeetsPlugin):
         by `self.patterns`.
         """
         if not self.patterns:
-            log.warn(u'[zero] no fields, nothing to do')
+            self._log.warn(u'no fields, nothing to do')
             return
 
         for field, patterns in self.patterns.items():
@@ -97,5 +97,5 @@ class ZeroPlugin(BeetsPlugin):
                 match = patterns is True
 
             if match:
-                log.debug(u'[zero] {0}: {1} -> None'.format(field, value))
+                self._log.debug(u'{0}: {1} -> None', field, value)
                 tags[field] = None
